@@ -57,10 +57,10 @@ from linaro_dashboard_bundle.io import DocumentIO
 
 from dashboard_app.helpers import BundleDeserializer
 from dashboard_app.managers import BundleManager, TestRunDenormalizationManager
-from dashboard_app.repositories import RepositoryItem 
+from dashboard_app.repositories import RepositoryItem
 from dashboard_app.repositories.data_report import DataReportRepository
 from dashboard_app.repositories.data_view import DataViewRepository
-from dashboard_app.signals import bundle_was_deserialized 
+from dashboard_app.signals import bundle_was_deserialized
 
 
 # Fix some django issues we ran into
@@ -737,7 +737,7 @@ class SoftwareSource(models.Model):
         help_text = _(u"Date and time of the commit (optional)"),
         verbose_name = _(u"Commit Timestamp")
     )
-    
+
     def __unicode__(self):
         return _(u"{project_name} from branch {branch_url} at revision {branch_revision}").format(
             project_name=self.project_name, branch_url=self.branch_url, branch_revision=self.branch_revision)
@@ -824,6 +824,11 @@ class TestRun(models.Model):
                       "with lots of tests results from 1972")
     )
 
+    microseconds = models.BigIntegerField(
+        blank = True,
+        null = True
+    )
+
     # Software Context
 
     sw_image_desc = models.CharField(
@@ -927,6 +932,27 @@ class TestRun(models.Model):
             self._cached_summary_results = self._get_summary_results()
         return self._cached_summary_results
 
+    # test_duration property
+
+    def _get_test_duration(self):
+        if self.microseconds is None:
+            return None
+        else:
+            return datetime.timedelta(microseconds = self.microseconds)
+
+    def _set_test_duration(self, duration):
+        if duration is None:
+            self.microseconds = None
+        else:
+            if not isinstance(duration, datetime.timedelta):
+                raise TypeError("duration must be a datetime.timedelta() instance")
+            self.microseconds = (
+                duration.microseconds +
+                (duration.seconds * 10 ** 6) +
+                (duration.days * 24 * 60 * 60 * 10 ** 6))
+
+    test_duration = property(_get_test_duration, _set_test_duration)
+
     class Meta:
         ordering = ['-import_assigned_date']
 
@@ -983,7 +1009,7 @@ class Attachment(models.Model):
     mime_type = models.CharField(
         verbose_name = _(u"MIME type"),
         max_length = 64)
-    
+
     public_url = models.URLField(
         verbose_name = _(u"Public URL"),
         max_length = 512,
@@ -1027,11 +1053,30 @@ class Attachment(models.Model):
             self.content_type.model == 'testrun'):
             return True
 
+    def is_test_result_attachment(self):
+        if (self.content_type.app_label == 'dashboard_app' and
+            self.content_type.model == 'testresult'):
+            return True
+
     @property
     def test_run(self):
         if self.is_test_run_attachment():
             return self.content_object
 
+    @property
+    def test_result(self):
+        if self.is_test_result_attachment():
+            return self.content_object
+
+    @property
+    def bundle(self):
+        if self.is_test_result_attachment():
+            run = self.test_result.test_run
+        elif self.is_test_run_attachment():
+            run = self.test_run
+        return run.bundle
+
+
     @models.permalink
     def get_absolute_url(self):
         if self.is_test_run_attachment():
@@ -1040,6 +1085,13 @@ class Attachment(models.Model):
                      self.test_run.bundle.content_sha1,
                      self.test_run.analyzer_assigned_uuid,
                      self.pk])
+        elif self.is_test_result_attachment():
+            return ("dashboard_app.views.result_attachment_detail",
+                    [self.test_result.test_run.bundle.bundle_stream.pathname,
+                     self.test_result.test_run.bundle.content_sha1,
+                     self.test_result.test_run.analyzer_assigned_uuid,
+                     self.test_result.relative_index,
+                     self.pk])
 
 
 class TestResult(models.Model):
@@ -1168,6 +1220,10 @@ class TestResult(models.Model):
 
     attributes = generic.GenericRelation(NamedAttribute)
 
+    # Attachments
+
+    attachments = generic.GenericRelation(Attachment)
+
     # Duration property
 
     def _get_duration(self):
@@ -1738,7 +1794,7 @@ class TestRunFilter(models.Model):
 
     # given filter:
     # select from testrun
-    #  where testrun.bundle in filter.bundle_streams ^ accessible_bundles 
+    #  where testrun.bundle in filter.bundle_streams ^ accessible_bundles
     #    and testrun has attribute with key = key1 and value = value1
     #    and testrun has attribute with key = key2 and value = value2
     #    and               ...
